import { braces, parentheses, SparqlBuilder, } from "./sparql_shared_builders.js";
class SparqlQueryBuilder extends SparqlBuilder {
    OFFSET(number) {
        return this.number(number, "OFFSET");
    }
    LIMIT(number) {
        return this.number(number, "LIMIT");
    }
    ORDER_BY(strings, ...values) {
        return this.template(strings, values, "ORDER BY");
    }
    HAVING(strings, ...values) {
        return this.template(strings, values, "HAVING", parentheses);
    }
    GROUP_BY(strings, ...values) {
        return this.template(strings, values, "GROUP BY");
    }
    WHERE(strings, ...values) {
        return this.template(strings, values, "WHERE", braces);
    }
    FROM_NAMED(stringOrNamedNode) {
        return this.namedNode(stringOrNamedNode, "FROM NAMED");
    }
    FROM(stringOrNamedNode) {
        return this.namedNode(stringOrNamedNode, "FROM");
    }
    SELECT(strings, ...values) {
        return this.template(strings, values, "SELECT");
    }
    SELECT_DISTINCT(strings, ...values) {
        return this.template(strings, values, "SELECT DISTINCT");
    }
    SELECT_REDUCED(strings, ...values) {
        return this.template(strings, values, "SELECT REDUCED");
    }
    CONSTRUCT(strings, ...values) {
        return this.template(strings, values, "CONSTRUCT", braces);
    }
    CONSTRUCT_WHERE(strings, ...values) {
        return this.template(strings, values, "CONSTRUCT WHERE", braces);
    }
    ASK(strings, ...values) {
        return this.template(strings, values, "ASK", braces);
    }
    ASK_FROM(stringOrNamedNode) {
        return this.namedNode(stringOrNamedNode, "ASK\nFROM");
    }
    ASK_FROM_NAMED(stringOrNamedNode) {
        return this.namedNode(stringOrNamedNode, "ASK\nFROM NAMED");
    }
    ASK_WHERE(strings, ...values) {
        return this.template(strings, values, "ASK\nWHERE", braces);
    }
    DESCRIBE(strings, ...values) {
        return this.template(strings, values, "DESCRIBE");
    }
}
export const SELECT = Object.assign((strings, ...values) => new SparqlQueryBuilder().SELECT(strings, ...values), {
    DISTINCT: (strings, ...values) => new SparqlQueryBuilder().SELECT_DISTINCT(strings, ...values),
    REDUCED: (strings, ...values) => new SparqlQueryBuilder().SELECT_REDUCED(strings, ...values),
    get ALL() {
        return new SparqlQueryBuilder().SELECT `*`;
    },
});
export const CONSTRUCT = Object.assign((strings, ...values) => new SparqlQueryBuilder().CONSTRUCT(strings, ...values), {
    WHERE: (strings, ...values) => new SparqlQueryBuilder().CONSTRUCT_WHERE(strings, ...values),
});
export const ASK = Object.assign((strings, ...values) => new SparqlQueryBuilder().ASK(strings, ...values), {
    FROM: (stringOrNamedNode) => new SparqlQueryBuilder().ASK_FROM(stringOrNamedNode),
    FROM_NAMED: (stringOrNamedNode) => new SparqlQueryBuilder().ASK_FROM_NAMED(stringOrNamedNode),
    WHERE: (strings, ...values) => new SparqlQueryBuilder().ASK_WHERE(strings, ...values),
});
export const DESCRIBE = (strings, ...values) => new SparqlQueryBuilder().DESCRIBE(strings, ...values);
