import { DefaultGraph } from "../rdf.js";
import xsd from "../namespaces/xsd.js";
import { escape } from "./escape.js";
export const blankNode = (term) => {
    return `_:${term.value}`;
};
export const namedNode = (term) => {
    return `<${term.value}>`;
};
export const variable = (term) => {
    return `?${term.value}`;
};
export const literal = (term) => {
    const datatype = term.datatype.value;
    if (datatype === xsd.integer || datatype === xsd.boolean ||
        datatype === xsd.decimal) {
        return term.value;
    }
    const value = `"${escape(term.value)}"`;
    if (term.language) {
        return `${value}@${term.language}`;
    }
    if (datatype !== xsd.string) {
        return `${value}^^${namedNode(term.datatype)}`;
    }
    return value;
};
const quad = (term) => {
    const triple = `${stringify(term.subject)} ${stringify(term.predicate)} ${stringify(term.object)} .`;
    if (term.graph.equals(DefaultGraph.INSTANCE)) {
        return triple;
    }
    return `GRAPH ${stringify(term.graph)} { ${triple} }`;
};
export const stringify = (term) => {
    switch (term.termType) {
        case "BlankNode":
            return blankNode(term);
        case "NamedNode":
            return namedNode(term);
        case "Variable":
            return variable(term);
        case "Literal":
            return literal(term);
        case "Quad":
            return quad(term);
        default:
            throw new Error("Unknown RDF type found.");
    }
};
