"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryBuilder = void 0;
const mod_js_1 = require("../schema/mod.js");
const mod_js_2 = require("../sparql/mod.js");
const rdf_js_1 = require("../rdf.js");
const ldkit_js_1 = __importDefault(require("../namespaces/ldkit.js"));
const rdf_js_2 = __importDefault(require("../namespaces/rdf.js"));
const encoder_js_1 = require("../encoder.js");
const query_helper_js_1 = require("./query_helper.js");
class QueryBuilder {
    constructor(schema, context) {
        Object.defineProperty(this, "schema", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "schemaProperties", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "context", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "df", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "deleteQuery", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (iris) => {
                return (0, mod_js_2.DELETE) `
      ?s ?p ?o
    `.WHERE `
      ?s ?p ?o .
      VALUES ?s { ${iris.map(this.df.namedNode)} }
    `.build();
            }
        });
        this.schema = schema;
        this.schemaProperties = (0, mod_js_1.getSchemaProperties)(this.schema);
        this.context = context;
        this.df = new rdf_js_1.DataFactory();
    }
    getResourceSignature() {
        return this.df.quad(this.df.variable("iri"), this.df.namedNode(rdf_js_2.default.type), this.df.namedNode(ldkit_js_1.default.Resource));
    }
    getTypesSignature() {
        return this.df.quad(this.df.variable("iri"), this.df.namedNode(rdf_js_2.default.type), this.df.variable("iri_type"));
    }
    entitiesToQuads(entities) {
        const quadArrays = entities.map((entity) => (0, encoder_js_1.encode)(entity, this.schema, this.context));
        return [].concat(...quadArrays);
    }
    getShape(includeOptional = false, wrapOptional = true, omitRootTypes = false) {
        const mainVar = "iri";
        const conditions = [];
        const populateConditionsRecursive = (s, varPrefix) => {
            const rdfType = s["@type"];
            const properties = (0, mod_js_1.getSchemaProperties)(s);
            if (varPrefix !== "iri" || !omitRootTypes) {
                rdfType.forEach((type) => {
                    conditions.push(this.df.quad(this.df.variable(varPrefix), this.df.namedNode(rdf_js_2.default.type), this.df.namedNode(type)));
                });
            }
            Object.keys(properties).forEach((prop, index) => {
                const property = properties[prop];
                const isOptional = property["@optional"];
                if (!includeOptional && isOptional) {
                    return;
                }
                if (wrapOptional && isOptional) {
                    conditions.push((0, mod_js_2.sparql) `\nOPTIONAL {`);
                }
                conditions.push(this.df.quad(this.df.variable(varPrefix), this.df.namedNode(property["@id"]), this.df.variable(`${varPrefix}_${index}`)));
                if (typeof property["@context"] === "object") {
                    populateConditionsRecursive(property["@context"], `${varPrefix}_${index}`);
                }
                if (wrapOptional && isOptional) {
                    conditions.push((0, mod_js_2.sparql) `\n}\n`);
                }
            });
        };
        populateConditionsRecursive(this.schema, mainVar);
        return conditions;
    }
    countQuery() {
        const quads = this.getShape();
        return (0, mod_js_2.SELECT) `(count(?iri) as ?count)`.WHERE `${quads}`.build();
    }
    getQuery(where, limit = 1000) {
        const selectSubQuery = (0, mod_js_2.SELECT) `
      ${this.df.variable("iri")}
    `.WHERE `
      ${this.getShape(false, true)}
      ${where}
    `.LIMIT(limit).build();
        const query = (0, mod_js_2.CONSTRUCT) `
      ${this.getResourceSignature()}
      ${this.getTypesSignature()}
      ${this.getShape(true, false, true)}
    `.WHERE `
      ${this.getTypesSignature()}
      ${this.getShape(true, true, true)}
      {
        ${selectSubQuery}
      }
    `.build();
        return query;
    }
    getByIrisQuery(iris) {
        const query = (0, mod_js_2.CONSTRUCT) `
      ${this.getResourceSignature()}
      ${this.getTypesSignature()}
      ${this.getShape(true, false, true)}
    `.WHERE `
      ${this.getTypesSignature()}
      ${this.getShape(true, true, true)}
      VALUES ?iri {
        ${iris.map(this.df.namedNode)}
      }
    `.build();
        return query;
    }
    insertQuery(entities) {
        const quads = this.entitiesToQuads(entities);
        return this.insertDataQuery(quads);
    }
    insertDataQuery(quads) {
        return mod_js_2.INSERT.DATA `${quads}`.build();
    }
    deleteDataQuery(quads) {
        return mod_js_2.DELETE.DATA `${quads}`.build();
    }
    updateQuery(entities) {
        const deleteQuads = [];
        const insertQuads = [];
        const whereQuads = [];
        entities.forEach((entity, index) => {
            const helper = new query_helper_js_1.QueryHelper(entity, this.schema, this.context, 1000 * index);
            deleteQuads.push(...helper.getDeleteQuads());
            insertQuads.push(...helper.getInsertQuads());
            whereQuads.push(...helper.getWhereQuads());
        });
        return (0, mod_js_2.DELETE) `${deleteQuads}`.INSERT `${insertQuads}`
            .WHERE `${deleteQuads}`.build();
    }
}
exports.QueryBuilder = QueryBuilder;
