"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sparql = void 0;
const rdf_js_1 = require("../rdf.js");
const xsd_js_1 = __importDefault(require("../namespaces/xsd.js"));
const stringify_js_1 = require("./stringify.js");
const sparql = (strings, ...values) => {
    let counter = 0;
    let result = "";
    for (const value of values) {
        result += strings[counter++];
        result += valueToString(value);
    }
    result += strings[counter];
    return result;
};
exports.sparql = sparql;
const isIterable = (obj) => {
    return Symbol.iterator in Object(obj);
};
const df = new rdf_js_1.DataFactory();
const valueToString = (value) => {
    if (typeof value === "undefined" || value === null) {
        return "";
    }
    if (typeof value === "string") {
        return value;
    }
    if (typeof value === "number") {
        const numberDataType = Number.isInteger(value) ? xsd_js_1.default.integer : xsd_js_1.default.decimal;
        return (0, stringify_js_1.stringify)(df.literal(value.toString(), df.namedNode(numberDataType)));
    }
    if (typeof value === "boolean") {
        return (0, stringify_js_1.stringify)(df.literal(value.toString(), df.namedNode(xsd_js_1.default.boolean)));
    }
    if (value instanceof Date) {
        return (0, stringify_js_1.stringify)(df.literal(value.toISOString(), df.namedNode(xsd_js_1.default.dateTime)));
    }
    if (isIterable(value)) {
        const [first, ...rest] = value;
        let result = valueToString(first);
        for (const part of rest) {
            result += `\n${valueToString(part)}`;
        }
        return result;
    }
    if (value.termType) {
        return (0, stringify_js_1.stringify)(value);
    }
    throw new Error("Not supported input type detected.");
};
