"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringify = exports.literal = exports.variable = exports.namedNode = exports.blankNode = void 0;
const rdf_js_1 = require("../rdf.js");
const xsd_js_1 = __importDefault(require("../namespaces/xsd.js"));
const escape_js_1 = require("./escape.js");
const blankNode = (term) => {
    return `_:${term.value}`;
};
exports.blankNode = blankNode;
const namedNode = (term) => {
    return `<${term.value}>`;
};
exports.namedNode = namedNode;
const variable = (term) => {
    return `?${term.value}`;
};
exports.variable = variable;
const literal = (term) => {
    const datatype = term.datatype.value;
    if (datatype === xsd_js_1.default.integer || datatype === xsd_js_1.default.boolean ||
        datatype === xsd_js_1.default.decimal) {
        return term.value;
    }
    const value = `"${(0, escape_js_1.escape)(term.value)}"`;
    if (term.language) {
        return `${value}@${term.language}`;
    }
    if (datatype !== xsd_js_1.default.string) {
        return `${value}^^${(0, exports.namedNode)(term.datatype)}`;
    }
    return value;
};
exports.literal = literal;
const quad = (term) => {
    const triple = `${(0, exports.stringify)(term.subject)} ${(0, exports.stringify)(term.predicate)} ${(0, exports.stringify)(term.object)} .`;
    if (term.graph.equals(rdf_js_1.DefaultGraph.INSTANCE)) {
        return triple;
    }
    return `GRAPH ${(0, exports.stringify)(term.graph)} { ${triple} }`;
};
const stringify = (term) => {
    switch (term.termType) {
        case "BlankNode":
            return (0, exports.blankNode)(term);
        case "NamedNode":
            return (0, exports.namedNode)(term);
        case "Variable":
            return (0, exports.variable)(term);
        case "Literal":
            return (0, exports.literal)(term);
        case "Quad":
            return quad(term);
        default:
            throw new Error("Unknown RDF type found.");
    }
};
exports.stringify = stringify;
