import type { Context, IQueryEngine, Iri, RDF } from "../rdf.js";
import { type SchemaInterface, type SchemaInterfaceIdentity, type SchemaPrototype } from "../schema/mod.js";
import type { Entity } from "./types.js";
/**
 * Lens lets you query and update RDF data via data schema using TypeScript native data types.
 *
 * https://ldkit.io/docs/components/lens
 *
 * @param schema data schema which extends `SchemaPrototype`
 * @param context optional `Context` - contains LDkit and query engine configuration
 * @param engine optional Query Engine
 * @returns Lens instance
 */
export declare const createLens: <T extends SchemaPrototype>(schema: T, context?: Context, engine?: IQueryEngine) => Lens<T, SchemaInterface<T>>;
/**
 * @deprecated
 * Use `createLens` instead
 */
export declare const createResource: <T extends SchemaPrototype>(schema: T, context?: Context, engine?: IQueryEngine) => Lens<T, SchemaInterface<T>>;
export declare class Lens<S extends SchemaPrototype, I = SchemaInterface<S>> {
    private readonly schema;
    private readonly context;
    private readonly engine;
    private readonly queryBuilder;
    constructor(schema: S, context?: Context, engine?: IQueryEngine);
    private decode;
    count(): Promise<number>;
    query(sparqlConstructQuery: string): Promise<I[]>;
    find(where?: string | RDF.Quad[], limit?: number): Promise<I[]>;
    findByIri(iri: Iri): Promise<I | undefined>;
    findByIris(iris: Iri[]): Promise<I[]>;
    private updateQuery;
    insert(...entities: Entity<I>[]): Promise<void>;
    insertData(...quads: RDF.Quad[]): Promise<void>;
    update(...entities: Entity<I>[]): Promise<void>;
    delete(...identities: SchemaInterfaceIdentity[] | Iri[]): Promise<void>;
    deleteData(...quads: RDF.Quad[]): Promise<void>;
}
