import { SparqlBuilder } from "./sparql_shared_builders.js";
import { type RDF } from "../rdf.js";
import { type SparqlValue } from "./sparql_tag.js";
type Builders<T extends keyof SparqlQueryBuilder> = Pick<SparqlQueryBuilder, T>;
declare class SparqlQueryBuilder extends SparqlBuilder {
    OFFSET(number: number): Builders<"build">;
    LIMIT(number: number): Builders<"build" | "OFFSET">;
    ORDER_BY(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "LIMIT">;
    HAVING(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "ORDER_BY" | "LIMIT">;
    GROUP_BY(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "HAVING" | "ORDER_BY" | "LIMIT">;
    WHERE(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "GROUP_BY" | "ORDER_BY" | "LIMIT">;
    FROM_NAMED(stringOrNamedNode: string | RDF.NamedNode<string>): Builders<"FROM_NAMED" | "WHERE">;
    FROM(stringOrNamedNode: string | RDF.NamedNode<string>): Builders<"FROM" | "FROM_NAMED" | "WHERE">;
    SELECT(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"FROM" | "FROM_NAMED" | "WHERE">;
    SELECT_DISTINCT(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"FROM" | "FROM_NAMED" | "WHERE">;
    SELECT_REDUCED(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"FROM" | "FROM_NAMED" | "WHERE">;
    CONSTRUCT(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"WHERE">;
    CONSTRUCT_WHERE(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "GROUP_BY" | "ORDER_BY" | "LIMIT">;
    ASK(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build">;
    ASK_FROM(stringOrNamedNode: string | RDF.NamedNode<string>): Builders<"FROM" | "FROM_NAMED" | "WHERE">;
    ASK_FROM_NAMED(stringOrNamedNode: string | RDF.NamedNode<string>): Builders<"FROM_NAMED" | "WHERE">;
    ASK_WHERE(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "GROUP_BY" | "ORDER_BY" | "LIMIT">;
    DESCRIBE(strings: TemplateStringsArray, ...values: SparqlValue[]): Builders<"build" | "FROM" | "FROM_NAMED" | "WHERE">;
}
export declare const SELECT: ((strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"WHERE" | "FROM_NAMED" | "FROM">) & {
    DISTINCT: (strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"WHERE" | "FROM_NAMED" | "FROM">;
    REDUCED: (strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"WHERE" | "FROM_NAMED" | "FROM">;
    readonly ALL: Builders<"WHERE" | "FROM_NAMED" | "FROM">;
};
export declare const CONSTRUCT: ((strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"WHERE">) & {
    WHERE: (strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"LIMIT" | "ORDER_BY" | "GROUP_BY" | "build">;
};
export declare const ASK: ((strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"build">) & {
    FROM: (stringOrNamedNode: string | RDF.NamedNode<string>) => Builders<"WHERE" | "FROM_NAMED" | "FROM">;
    FROM_NAMED: (stringOrNamedNode: string | RDF.NamedNode<string>) => Builders<"WHERE" | "FROM_NAMED">;
    WHERE: (strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"LIMIT" | "ORDER_BY" | "GROUP_BY" | "build">;
};
export declare const DESCRIBE: (strings: TemplateStringsArray, ...values: SparqlValue[]) => Builders<"WHERE" | "FROM_NAMED" | "FROM" | "build">;
export {};
